import {
  IDictionary, IHttpResponse, Interceptor, I18n, dayjs as moment, Config,
} from '@zeedhi/core';

export interface RequestLog {
  method: 'put' | 'post' | 'delete';
  url: string;
  row: IDictionary;
  operation: string;
  timestamp: number;
  route: string;
}

export interface RequestErrorLog {
  method: string;
  url: string;
  row: IDictionary;
  operation: string;
  timestamp: number;
  error: string;
  description: string;
  seen: boolean;
  objectName: string;
  date: string;
  dateTime: string;
}

const routesToLog = [
  'inventarios',
  'produtosInventario',
  'servicosCardapioEfetivos',
  'clientesDoServicoEfetivos',
  'produtosSolicitacao',
];

const routesToLogDisplayName = {
  inventarios: 'INVENTORY',
  produtosInventario: 'INVENTORY_PRODUCT',
  servicosCardapioEfetivos: 'SERVICE',
  clientesDoServicoEfetivos: 'CLIENT',
  produtosSolicitacao: 'SOLICITATION_PRODUCT',
};

const registerLogger = () => {
  Interceptor.addResponseSuccess((response: IHttpResponse<{ data: IDictionary }>) => {
    // se estiver online não precisamos salvar no histórico
    // if (navigator.onLine) return response;

    // const { statusText } = response;
    const { method } = response.config;
    const url = response.config.url || '';
    const route = url.split('/')[0];
    // cria um log ao adicionar/modificar/deletar um recurso
    if ((method === 'post' || method === 'put' || method === 'delete') && routesToLog.includes(route)) {
      const timestamp = Date.now();
      let operation = '';
      switch (method) {
        case 'post':
          operation = 'create';
          break;
        case 'put':
          operation = 'update';
          break;
        case 'delete':
          operation = 'delete';
          break;
        default:
          break;
      }

      let row = response.data.data;
      if (method === 'put' || method === 'delete' || method === 'post') {
        row = JSON.parse(response.config.data);
      } else row = [];
      const newLog: RequestLog = {
        method, url, row, operation, timestamp, route,
      };
      const item = localStorage.getItem('request-log') || '[]';
      const logs: RequestLog[] = JSON.parse(item);

      logs.push(newLog);

      // salva o log na localstorage
      const stringified = JSON.stringify(logs);
      localStorage.setItem('request-log', stringified);
    }

    return response;
  });
};

const possibleMessages = [
  {
    message: 'ERROR_CHANGE_QUANTITY_EXPORT',
    hasParams: true,
  },
  {
    message: 'ERROR_DELETE_PRODUCT_EXPORT',
    hasParams: true,
  },
  {
    message: 'PRODUCT_EXISTS_PARAM',
    hasParams: true,
  },
  {
    message: 'DELETE_NOT_FOUND',
    hasParams: false,
  },
  {
    message: 'EDIT_NOT_FOUND',
    hasParams: true,
  },
  {
    message: 'INVENTORY_DONT_EXIST',
    hasParams: true,
  },
  {
    message: 'INVENTORY_NOT_INSERTED_EXPORT',
    hasParams: true,
  },
  {
    message: 'DELETE',
    hasParams: false,
  },
  {
    message: 'INSERT',
    hasParams: false,
  },
  {
    message: 'UPDATE',
    hasParams: false,
  },
];

const registerErrorLogger = (response: IDictionary) => {
  const item = localStorage.getItem('request-error-log') || '[]';
  const errorLogs: RequestErrorLog[] = JSON.parse(item);

  // const statusText = response?.statusText || '';
  const { method } = response?.config;
  const url = response?.config?.url || '';
  const row = response?.data?.data || {};
  const timestamp = Date.now();
  // const operation = statusText?.slice(0, -1).toLowerCase();
  const route = url.split('/')[0] as keyof typeof routesToLogDisplayName;
  const objectName = I18n.translate(`scm::${routesToLogDisplayName[route]}` || '');
  const operation = url.split('/')[1] || '';
  const { error } = response?.data || '';
  let description = error;
  // if (error?.length !== 0 && error?.error !== null) {
  if (error?.length !== 0) {
    const mensagem = error;
    possibleMessages.some((el: IDictionary) => {
      if (mensagem.indexOf(el.message) >= 0) {
        if (el.hasParams) {
          const param = mensagem.split(':');
          description = I18n.translate(`scm::${el.message}`, { PARAM1: param[1] });
        } else {
          description = I18n.translate(`scm::${el.message}`);
        }
        return true;
      }
      return false;
    });
  }
  const date = moment(timestamp).format(Config.displayFormat);
  // const dateTime = moment(timestamp).format(Config.dateFormat);
  const dateTime = moment(timestamp).format(
    (localStorage.getItem('LOGIN_LANGUAGE') !== 'en_us') ? 'DD/MM/YYYY HH:mm:ss' : 'MM/DD/YYYY hh:mm:ss A',
  );
  const newLog: RequestErrorLog = {
    method, url, row, operation, timestamp, error, description, seen: false, objectName, date, dateTime,
  };
  errorLogs.push(newLog);

  // salva o log na localstorage
  const stringified = JSON.stringify(errorLogs);
  localStorage.setItem('request-error-log', stringified);
};

export { registerLogger, registerErrorLogger };
